#include <Arduino.h>
#include <WiFi.h>
#include <ArduinoJson.h>

const int scanIterations = 5;
const int maxChannel = 14; // Maximum number of Wi-Fi channels in the 2.4 GHz band
int channelUsage[maxChannel] = {0};

void setup() {
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  Serial.println("Setup complete. Starting Wi-Fi channel analysis.");
}

void loop() {
  Serial.println("Scanning...");

  for (int iteration = 0; iteration < scanIterations; iteration++) {
    int numberOfNetworks = WiFi.scanNetworks();

    for (int networkIndex = 0; networkIndex < numberOfNetworks; networkIndex++) {
      int channel = WiFi.channel(networkIndex);
      if (channel > 0 && channel <= maxChannel) {
        channelUsage[channel - 1]++;
      }
    }

    delay(1000);
  }

  // Create a JSON object to store channel usage data
  StaticJsonDocument<256> jsonDocument;
  JsonObject channelUsageJson = jsonDocument.to<JsonObject>();

  for (int channelIndex = 0; channelIndex < maxChannel; channelIndex++) {
    String channel = String(channelIndex + 1);
    channelUsageJson[channel] = channelUsage[channelIndex];
    channelUsage[channelIndex] = 0; // Reset channel usage counter for the next analysis
  }

  // Serialize and print the JSON object
  Serial.println("Analysis complete. Channel usage:");
  serializeJsonPretty(channelUsageJson, Serial);
  Serial.println("\nRescanning in 5 seconds...");
  delay(5000);
}
